@@ -210,7 +210,7 @@ igraph_error_t igraph_copy(igraph_t *to, const igraph_t *from) {
     IGRAPH_CHECK(igraph_i_property_cache_copy(to->cache, from->cache));
     IGRAPH_FINALLY(igraph_i_property_cache_destroy, to->cache);
 
-    IGRAPH_I_ATTRIBUTE_COPY(to, from, 1, 1, 1); /* does IGRAPH_CHECK */
+    IGRAPH_I_ATTRIBUTE_COPY(to, from, true, true, true); /* does IGRAPH_CHECK */
 
     IGRAPH_FINALLY_CLEAN(8);
     return IGRAPH_SUCCESS;
@@ -509,9 +509,9 @@ igraph_error_t igraph_delete_edges(igraph_t *graph, igraph_es_t edges) {
 
     for (IGRAPH_EIT_RESET(eit); !IGRAPH_EIT_END(eit); IGRAPH_EIT_NEXT(eit)) {
         igraph_integer_t e = IGRAPH_EIT_GET(eit);
-        if (mark[e] == 0) {
+        if (! mark[e]) {
             edges_to_remove++;
-            mark[e]++;
+            mark[e] = true;
         }
     }
     remaining_edges = no_of_edges - edges_to_remove;
@@ -525,7 +525,7 @@ igraph_error_t igraph_delete_edges(igraph_t *graph, igraph_es_t edges) {
 
     /* Actually remove the edges, move from pos i to pos j in newfrom/newto */
     for (i = 0, j = 0; j < remaining_edges; i++) {
-        if (mark[i] == 0) {
+        if (! mark[i]) {
             VECTOR(newfrom)[j] = VECTOR(graph->from)[i];
             VECTOR(newto)[j] = VECTOR(graph->to)[i];
             j++;
@@ -545,7 +545,7 @@ igraph_error_t igraph_delete_edges(igraph_t *graph, igraph_es_t edges) {
         igraph_vector_int_t idx;
         IGRAPH_VECTOR_INT_INIT_FINALLY(&idx, remaining_edges);
         for (i = 0, j = 0; i < no_of_edges; i++) {
-            if (mark[i] == 0) {
+            if (! mark[i]) {
                 VECTOR(idx)[j++] = i;
             }
         }
@@ -622,7 +622,9 @@ igraph_error_t igraph_delete_edges(igraph_t *graph, igraph_es_t edges) {
  *     may contain the same ID more than once.
  * \param idx An optional pointer to a vector that provides the mapping from
  *     the vertex IDs \em before the removal to the vertex IDs \em after
- *     the removal. You can supply \c NULL here if you are not interested.
+ *     the removal, \em plus one. Zero is used to represent vertices that were
+ *     removed during the operation. You can supply \c NULL here if you are not
+ *     interested.
  * \param invidx An optional pointer to a vector that provides the mapping from
  *     the vertex IDs \em after the removal to the vertex IDs \em before
  *     the removal. You can supply \c NULL here if you are not interested.
@@ -883,9 +885,16 @@ igraph_error_t igraph_i_neighbors(const igraph_t *graph, igraph_vector_int_t *ne
         igraph_neimode_t mode, igraph_loops_t loops, igraph_multiple_t multiple) {
 #define DEDUPLICATE_IF_NEEDED(vertex, n)                                                 \
     if (should_filter_duplicates) {                                                        \
-        if ((loops == IGRAPH_NO_LOOPS && vertex == pnode) ||                               \
-                (loops == IGRAPH_LOOPS_ONCE && vertex == pnode && last_added == pnode) ||  \
-                (multiple == IGRAPH_NO_MULTIPLE && vertex == last_added)) {                \
+        if (                                                                               \
+            (loops == IGRAPH_NO_LOOPS && vertex == pnode) ||                               \
+            (loops == IGRAPH_LOOPS_ONCE && vertex == pnode && last_added == pnode)         \
+        ) {                                                                                \
+            length -= n;                                                                   \
+            if (loops == IGRAPH_LOOPS_ONCE) {                                              \
+                last_added = -1;                                                           \
+            }                                                                              \
+            continue;                                                                      \
+        } else if (multiple == IGRAPH_NO_MULTIPLE && vertex == last_added) {               \
             length -= n;                                                                   \
             continue;                                                                      \
         } else {                                                                           \
@@ -955,6 +964,7 @@ igraph_error_t igraph_i_neighbors(const igraph_t *graph, igraph_vector_int_t *ne
                 VECTOR(*neis)[idx++] = to;
             }
         }
+
         if (mode & IGRAPH_IN) {
             j = VECTOR(graph->is)[node + 1];
             for (i = VECTOR(graph->is)[node]; i < j; i++) {
@@ -998,6 +1008,9 @@ igraph_error_t igraph_i_neighbors(const igraph_t *graph, igraph_vector_int_t *ne
                 if (should_filter_duplicates && ((loops == IGRAPH_LOOPS_ONCE && n1 == pnode && last_added == pnode) ||
                         (multiple == IGRAPH_NO_MULTIPLE))) {
                     length--;
+                    if (loops == IGRAPH_LOOPS_ONCE) {
+                        last_added = -1;
+                    }
                     continue;
                 }
                 VECTOR(*neis)[idx++] = n2;
@@ -1092,6 +1105,67 @@ igraph_bool_t igraph_is_directed(const igraph_t *graph) {
     return graph->directed;
 }
 
+/**
+ * \ingroup interface
+ * \function igraph_degree_1
+ * \brief The degree of of a single vertex in the graph.
+ *
+ * This function calculates the in-, out- or total degree of a single vertex.
+ * For a single vertex, it is more efficient than calling \ref igraph_degree().
+ *
+ * \param graph The graph.
+ * \param deg Pointer to the integer where the computed degree will be stored.
+ * \param vid The vertex for which the degree will be calculated.
+ * \param mode Defines the type of the degree for directed graphs. Valid modes are:
+ *        \c IGRAPH_OUT, out-degree;
+ *        \c IGRAPH_IN, in-degree;
+ *        \c IGRAPH_ALL, total degree (sum of the in- and out-degree).
+ *        This parameter is ignored for undirected graphs.
+ * \param loops Boolean, gives whether the self-loops should be
+ *        counted.
+ * \return Error code.
+ *
+ * \sa \ref igraph_degree() to compute the degree of several vertices at once.
+ *
+ * Time complexity: O(1) if \p loops is \c true, and
+ * O(d) otherwise, where d is the degree.
+ */
+igraph_error_t igraph_degree_1(const igraph_t *graph, igraph_integer_t *deg,
+                               igraph_integer_t vid, igraph_neimode_t mode, igraph_bool_t loops) {
+
+    if (!igraph_is_directed(graph)) {
+        mode = IGRAPH_ALL;
+    }
+
+    *deg = 0;
+    if (mode & IGRAPH_OUT) {
+        *deg += (VECTOR(graph->os)[vid + 1] - VECTOR(graph->os)[vid]);
+    }
+    if (mode & IGRAPH_IN) {
+        *deg += (VECTOR(graph->is)[vid + 1] - VECTOR(graph->is)[vid]);
+    }
+    if (! loops) {
+        /* When loops should not be counted, we remove their contribution from the
+         * previously computed degree. */
+        if (mode & IGRAPH_OUT) {
+            for (igraph_integer_t i = VECTOR(graph->os)[vid]; i < VECTOR(graph->os)[vid + 1]; i++) {
+                if (VECTOR(graph->to)[ VECTOR(graph->oi)[i] ] == vid) {
+                    *deg -= 1;
+                }
+            }
+        }
+        if (mode & IGRAPH_IN) {
+            for (igraph_integer_t i = VECTOR(graph->is)[vid]; i < VECTOR(graph->is)[vid + 1]; i++) {
+                if (VECTOR(graph->from)[ VECTOR(graph->ii)[i] ] == vid) {
+                    *deg -= 1;
+                }
+            }
+        }
+    }
+
+    return IGRAPH_SUCCESS;
+}
+
 /**
  * \ingroup interface
  * \function igraph_degree
@@ -1129,7 +1203,8 @@ igraph_bool_t igraph_is_directed(const igraph_t *graph) {
  * d is their (average) degree.
  *
  * \sa \ref igraph_strength() for the version that takes into account
- * edge weights.
+ * edge weights; \ref igraph_degree_1() to efficiently compute the
+ * degree of a single vertex.
  *
  * \example examples/simple/igraph_degree.c
  */
@@ -1259,11 +1334,11 @@ igraph_error_t igraph_degree(const igraph_t *graph, igraph_vector_int_t *res,
         igraph_integer_t start2 = VECTOR(graph->is)[xto]; \
         igraph_integer_t end2 = VECTOR(graph->is)[xto+1]; \
         igraph_integer_t N2 = end2; \
-        igraph_integer_t *nullptr = 0; \
+        igraph_integer_t *nullpointer = NULL; \
         if (end-start < end2-start2) { \
-            BINSEARCH(start, end, xto, graph->oi, graph->to, N, eid, nullptr); \
+            BINSEARCH(start, end, xto, graph->oi, graph->to, N, eid, nullpointer); \
         } else { \
-            BINSEARCH(start2, end2, xfrom, graph->ii, graph->from, N2, eid, nullptr); \
+            BINSEARCH(start2, end2, xfrom, graph->ii, graph->from, N2, eid, nullpointer); \
         } \
     } while (0)
 
@@ -1278,26 +1353,26 @@ igraph_error_t igraph_degree(const igraph_t *graph, igraph_vector_int_t *res,
  * \function igraph_get_eid
  * \brief Get the edge ID from the end points of an edge.
  *
- * For undirected graphs \c pfrom and \c pto are exchangeable.
+ * For undirected graphs \c from and \c to are exchangeable.
  *
  * \param graph The graph object.
  * \param eid Pointer to an integer, the edge ID will be stored here.
- * \param pfrom The starting point of the edge.
- * \param pto The end point of the edge.
+ * \param from The starting point of the edge.
+ * \param to The end point of the edge.
  * \param directed Logical constant, whether to search for directed
  *        edges in a directed graph. Ignored for undirected graphs.
  * \param error Logical scalar, whether to report an error if the edge
  *        was not found. If it is false, then -1 will be assigned to \p eid.
- *        Note that invalid vertex IDs in input arguments (\p pfrom or \p pto)
+ *        Note that invalid vertex IDs in input arguments (\p from or \p to)
  *        always return an error code.
  * \return Error code.
  * \sa \ref igraph_edge() for the opposite operation, \ref igraph_get_all_eids_between()
  *     to retrieve all edge IDs between a pair of vertices.
  *
  * Time complexity: O(log (d)), where d is smaller of the out-degree
- * of \c pfrom and in-degree of \c pto if \p directed is true. If \p directed
+ * of \c from and in-degree of \c to if \p directed is true. If \p directed
  * is false, then it is O(log(d)+log(d2)), where d is the same as before and
- * d2 is the minimum of the out-degree of \c pto and the in-degree of \c pfrom.
+ * d2 is the minimum of the out-degree of \c to and the in-degree of \c from.
  *
  * \example examples/simple/igraph_get_eid.c
  *
@@ -1305,14 +1380,13 @@ igraph_error_t igraph_degree(const igraph_t *graph, igraph_vector_int_t *res,
  */
 
 igraph_error_t igraph_get_eid(const igraph_t *graph, igraph_integer_t *eid,
-                   igraph_integer_t pfrom, igraph_integer_t pto,
+                   igraph_integer_t from, igraph_integer_t to,
                    igraph_bool_t directed, igraph_bool_t error) {
 
-    igraph_integer_t from = pfrom, to = pto;
-    igraph_integer_t nov = igraph_vcount(graph);
+    igraph_integer_t no_of_nodes = igraph_vcount(graph);
 
-    if (from < 0 || to < 0 || from > nov - 1 || to > nov - 1) {
-        IGRAPH_ERROR("cannot get edge ID", IGRAPH_EINVVID);
+    if (from < 0 || to < 0 || from >= no_of_nodes || to >= no_of_nodes) {
+        IGRAPH_ERROR("Cannot get edge ID.", IGRAPH_EINVVID);
     }
 
     *eid = -1;
@@ -1555,32 +1629,18 @@ igraph_error_t igraph_get_all_eids_between(
 igraph_error_t igraph_incident(const igraph_t *graph, igraph_vector_int_t *eids, igraph_integer_t pnode,
         igraph_neimode_t mode) {
     if (!igraph_is_directed(graph) || mode == IGRAPH_ALL) {
-        return igraph_i_incident(graph, eids, pnode, mode, IGRAPH_LOOPS_TWICE, IGRAPH_MULTIPLE);
+        return igraph_i_incident(graph, eids, pnode, mode, IGRAPH_LOOPS_TWICE);
     } else {
-        return igraph_i_incident(graph, eids, pnode, mode, IGRAPH_LOOPS_ONCE, IGRAPH_MULTIPLE);
+        return igraph_i_incident(graph, eids, pnode, mode, IGRAPH_LOOPS_ONCE);
     }
 }
 
 igraph_error_t igraph_i_incident(const igraph_t *graph, igraph_vector_int_t *eids, igraph_integer_t pnode,
-        igraph_neimode_t mode, igraph_loops_t loops, igraph_multiple_t multiple) {
-#define DEDUPLICATE_IF_NEEDED(vertex, n)                                                 \
-    if (should_filter_duplicates) {                                                        \
-        if ((loops == IGRAPH_NO_LOOPS && vertex == pnode) ||                               \
-                (loops == IGRAPH_LOOPS_ONCE && vertex == pnode && last_added == pnode) ||  \
-                (multiple == IGRAPH_NO_MULTIPLE && vertex == last_added)) {                \
-            length -= n;                                                                   \
-            continue;                                                                      \
-        } else {                                                                           \
-            last_added = vertex;                                                           \
-        }                                                                                  \
-    }
+        igraph_neimode_t mode, igraph_loops_t loops) {
     igraph_integer_t length = 0, idx = 0;
     igraph_integer_t i, j;
-
     igraph_integer_t node = pnode;
-    igraph_integer_t last_added = -1;
-
-    igraph_bool_t should_filter_duplicates;
+    igraph_bool_t directed = igraph_is_directed(graph);
 
     if (node < 0 || node > igraph_vcount(graph) - 1) {
         IGRAPH_ERROR("Given vertex is not in the graph.", IGRAPH_EINVVID);
@@ -1590,7 +1650,7 @@ igraph_error_t igraph_i_incident(const igraph_t *graph, igraph_vector_int_t *eid
         IGRAPH_ERROR("Mode should be either IGRAPH_OUT, IGRAPH_IN or IGRAPH_ALL.", IGRAPH_EINVMODE);
     }
 
-    if (!igraph_is_directed(graph)) {
+    if (!directed) {
         mode = IGRAPH_ALL;
     }
 
@@ -1618,29 +1678,32 @@ igraph_error_t igraph_i_incident(const igraph_t *graph, igraph_vector_int_t *eid
      * an easy job. If we have requested both, we need to merge the two lists
      * to ensure that the output is sorted by the vertex IDs of the "other"
      * endpoint of the affected edges */
-    if (!igraph_is_directed(graph) || mode != IGRAPH_ALL) {
+    if (!directed || mode != IGRAPH_ALL) {
         /* We did not ask for both directions; this is the easy case */
 
-        should_filter_duplicates = !(multiple == IGRAPH_MULTIPLE &&
-                ((!igraph_is_directed(graph) && loops == IGRAPH_LOOPS_TWICE) ||
-                 (igraph_is_directed(graph) && loops != IGRAPH_NO_LOOPS)));
-
         if (mode & IGRAPH_OUT) {
             j = VECTOR(graph->os)[node + 1];
             for (i = VECTOR(graph->os)[node]; i < j; i++) {
                 igraph_integer_t edge = VECTOR(graph->oi)[i];
                 igraph_integer_t other = VECTOR(graph->to)[edge];
-                DEDUPLICATE_IF_NEEDED(other, 1);
-                VECTOR(*eids)[idx++] = edge;
+                if (loops == IGRAPH_NO_LOOPS && other == pnode) {
+                    length--;
+                } else {
+                    VECTOR(*eids)[idx++] = edge;
+                }
             }
         }
+
         if (mode & IGRAPH_IN) {
             j = VECTOR(graph->is)[node + 1];
             for (i = VECTOR(graph->is)[node]; i < j; i++) {
                 igraph_integer_t edge = VECTOR(graph->ii)[i];
                 igraph_integer_t other = VECTOR(graph->from)[edge];
-                DEDUPLICATE_IF_NEEDED(other, 1);
-                VECTOR(*eids)[idx++] = edge;
+                if ((loops == IGRAPH_NO_LOOPS || (loops == IGRAPH_LOOPS_ONCE && !directed)) && other == pnode) {
+                    length--;
+                } else {
+                    VECTOR(*eids)[idx++] = edge;
+                }
             }
         }
     } else {
@@ -1652,9 +1715,7 @@ igraph_error_t igraph_i_incident(const igraph_t *graph, igraph_vector_int_t *eid
         igraph_integer_t i2 = VECTOR(graph->is)[node];
         igraph_integer_t eid1, eid2;
         igraph_integer_t n1, n2;
-
-        should_filter_duplicates = !(multiple == IGRAPH_MULTIPLE &&
-                loops == IGRAPH_LOOPS_TWICE);
+        igraph_bool_t seen_loop_edge = 0;
 
         while (i1 < j1 && i2 < j2) {
             eid1 = VECTOR(graph->oi)[i1];
@@ -1663,37 +1724,44 @@ igraph_error_t igraph_i_incident(const igraph_t *graph, igraph_vector_int_t *eid
             n2 = VECTOR(graph->from)[eid2];
             if (n1 < n2) {
                 i1++;
-                DEDUPLICATE_IF_NEEDED(n1, 1);
                 VECTOR(*eids)[idx++] = eid1;
             } else if (n1 > n2) {
                 i2++;
-                DEDUPLICATE_IF_NEEDED(n2, 1);
                 VECTOR(*eids)[idx++] = eid2;
-            } else {
+            } else if (n1 != pnode) {
+                /* multiple edge */
                 i1++;
                 i2++;
-                DEDUPLICATE_IF_NEEDED(n2, 2);
                 VECTOR(*eids)[idx++] = eid1;
-                if (should_filter_duplicates && ((loops == IGRAPH_LOOPS_ONCE && n1 == pnode && last_added == pnode) ||
-                        (multiple == IGRAPH_NO_MULTIPLE))) {
-                    length--;
-                    continue;
-                }
                 VECTOR(*eids)[idx++] = eid2;
+            } else {
+                /* loop edge */
+                i1++;
+                i2++;
+                if (loops == IGRAPH_NO_LOOPS) {
+                    length -= 2;
+                } else if (loops == IGRAPH_LOOPS_ONCE) {
+                    length--;
+                    if (!seen_loop_edge) {
+                        VECTOR(*eids)[idx++] = eid1;
+                    } else {
+                        VECTOR(*eids)[idx++] = eid2;
+                    }
+                    seen_loop_edge = !seen_loop_edge;
+                } else {
+                    VECTOR(*eids)[idx++] = eid1;
+                    VECTOR(*eids)[idx++] = eid2;
+                }
             }
         }
 
         while (i1 < j1) {
             eid1 = VECTOR(graph->oi)[i1++];
-            igraph_integer_t to = VECTOR(graph->to)[eid1];
-            DEDUPLICATE_IF_NEEDED(to, 1);
             VECTOR(*eids)[idx++] = eid1;
         }
 
         while (i2 < j2) {
             eid2 = VECTOR(graph->ii)[i2++];
-            igraph_integer_t from = VECTOR(graph->from)[eid2];
-            DEDUPLICATE_IF_NEEDED(from, 1);
             VECTOR(*eids)[idx++] = eid2;
         }
     }
