@@ -464,10 +464,10 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
                         end = true;
                     } else if (edge_color1 || edge_compat_fn) {
                         igraph_integer_t eid1, eid2;
-                        igraph_get_eid(graph1, &eid1, node, cand1, /*directed=*/ 1,
-                                       /*error=*/ 1);
-                        igraph_get_eid(graph2, &eid2, node2, cand2, /*directed=*/ 1,
-                                       /*error=*/ 1);
+                        igraph_get_eid(graph1, &eid1, node, cand1, IGRAPH_DIRECTED,
+                                       /*error=*/ true);
+                        igraph_get_eid(graph2, &eid2, node2, cand2, IGRAPH_DIRECTED,
+                                       /*error=*/ true);
                         if (edge_color1 && VECTOR(*edge_color1)[eid1] !=
                             VECTOR(*edge_color2)[eid2]) {
                             end = true;
@@ -496,10 +496,10 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
                         end = true;
                     } else if (edge_color1 || edge_compat_fn) {
                         igraph_integer_t eid1, eid2;
-                        igraph_get_eid(graph1, &eid1, cand1, node, /*directed=*/ 1,
-                                       /*error=*/ 1);
-                        igraph_get_eid(graph2, &eid2, cand2, node2, /*directed=*/ 1,
-                                       /*error=*/ 1);
+                        igraph_get_eid(graph1, &eid1, cand1, node, IGRAPH_DIRECTED,
+                                       /*error=*/ true);
+                        igraph_get_eid(graph2, &eid2, cand2, node2, IGRAPH_DIRECTED,
+                                       /*error=*/ true);
                         if (edge_color1 && VECTOR(*edge_color1)[eid1] !=
                             VECTOR(*edge_color2)[eid2]) {
                             end = true;
@@ -528,10 +528,10 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
                         end = true;
                     } else if (edge_color1 || edge_compat_fn) {
                         igraph_integer_t eid1, eid2;
-                        igraph_get_eid(graph1, &eid1, node2, cand1, /*directed=*/ 1,
-                                       /*error=*/ 1);
-                        igraph_get_eid(graph2, &eid2, node, cand2, /*directed=*/ 1,
-                                       /*error=*/ 1);
+                        igraph_get_eid(graph1, &eid1, node2, cand1, IGRAPH_DIRECTED,
+                                       /*error=*/ true);
+                        igraph_get_eid(graph2, &eid2, node, cand2, IGRAPH_DIRECTED,
+                                       /*error=*/ true);
                         if (edge_color1 && VECTOR(*edge_color1)[eid1] !=
                             VECTOR(*edge_color2)[eid2]) {
                             end = true;
@@ -560,10 +560,10 @@ igraph_error_t igraph_get_isomorphisms_vf2_callback(
                         end = true;
                     } else if (edge_color1 || edge_compat_fn) {
                         igraph_integer_t eid1, eid2;
-                        igraph_get_eid(graph1, &eid1, cand1, node2, /*directed=*/ 1,
-                                       /*error=*/ 1);
-                        igraph_get_eid(graph2, &eid2, cand2, node, /*directed=*/ 1,
-                                       /*error=*/ 1);
+                        igraph_get_eid(graph1, &eid1, cand1, node2, IGRAPH_DIRECTED,
+                                       /*error=*/ true);
+                        igraph_get_eid(graph2, &eid2, cand2, node, IGRAPH_DIRECTED,
+                                       /*error=*/ true);
                         if (edge_color1 && VECTOR(*edge_color1)[eid1] !=
                             VECTOR(*edge_color2)[eid2]) {
                             end = true;
@@ -858,6 +858,7 @@ static igraph_error_t igraph_i_count_isomorphisms_vf2_cb(
  * This function counts the number of isomorphic mappings between two
  * graphs. It uses the generic \ref igraph_get_isomorphisms_vf2_callback()
  * function.
+ *
  * \param graph1 The first input graph, may be directed or undirected.
  * \param graph2 The second input graph, it must have the same
  *   directedness as \p graph1, or an error will be reported.
@@ -884,6 +885,8 @@ static igraph_error_t igraph_i_count_isomorphisms_vf2_cb(
  *   \p edge_compat_fn.
  * \return Error code.
  *
+ * \sa igraph_count_automorphisms()
+ *
  * Time complexity: exponential.
  */
 
@@ -1362,10 +1365,10 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
                         end = true;
                     } else if (edge_color1 || edge_compat_fn) {
                         igraph_integer_t eid1, eid2;
-                        igraph_get_eid(graph1, &eid1, node2, cand1, /*directed=*/ 1,
-                                       /*error=*/ 1);
-                        igraph_get_eid(graph2, &eid2, node, cand2, /*directed=*/ 1,
-                                       /*error=*/ 1);
+                        igraph_get_eid(graph1, &eid1, node2, cand1, IGRAPH_DIRECTED,
+                                       /*error=*/ true);
+                        igraph_get_eid(graph2, &eid2, node, cand2, IGRAPH_DIRECTED,
+                                       /*error=*/ true);
                         if (edge_color1 && VECTOR(*edge_color1)[eid1] !=
                             VECTOR(*edge_color2)[eid2]) {
                             end = true;
@@ -1394,10 +1397,10 @@ igraph_error_t igraph_get_subisomorphisms_vf2_callback(
                         end = true;
                     } else if (edge_color1 || edge_compat_fn) {
                         igraph_integer_t eid1, eid2;
-                        igraph_get_eid(graph1, &eid1, cand1, node2, /*directed=*/ 1,
-                                       /*error=*/ 1);
-                        igraph_get_eid(graph2, &eid2, cand2, node, /*directed=*/ 1,
-                                       /*error=*/ 1);
+                        igraph_get_eid(graph1, &eid1, cand1, node2, IGRAPH_DIRECTED,
+                                       /*error=*/ true);
+                        igraph_get_eid(graph2, &eid2, cand2, node, IGRAPH_DIRECTED,
+                                       /*error=*/ true);
                         if (edge_color1 && VECTOR(*edge_color1)[eid1] !=
                             VECTOR(*edge_color2)[eid2]) {
                             end = true;
