@@ -44,6 +44,29 @@ class SubgraphTests(unittest.TestCase):
 
 
 class DecompositionTests(unittest.TestCase):
+    def testDecomposeUndirected(self):
+        g = Graph([(0, 1), (1, 2), (2, 3)], n=4, directed=False)
+        components = g.decompose()
+
+        assert len(components) == 1
+        assert components[0].isomorphic(g)
+
+        g = Graph.Full(5) + Graph.Full(3)
+        components = g.decompose()
+
+        assert len(components) == 2
+        assert components[0].isomorphic(Graph.Full(5))
+        assert components[1].isomorphic(Graph.Full(3))
+
+    def testDecomposeDirected(self):
+        g = Graph([(0, 1), (1, 2), (2, 3)], n=4, directed=True)
+        components = g.decompose()
+
+        g1 = Graph(1, directed=True)
+        assert len(components) == 4
+        for component in components:
+            assert component.isomorphic(g1)
+
     def testKCores(self):
         g = Graph(
             11,
@@ -140,6 +163,17 @@ class VertexClusteringTests(unittest.TestCase):
         self.assertTrue(clg.vs["string"] == ["aaa", "bbc", "ccab"])
         self.assertTrue(clg.vs["int"] == [41, 64, 47])
 
+    def testSizesWithNone(self):
+        cl = VertexClustering(self.graph, [0, 0, 0, None, 1, 1, 2, None, 2, None])
+        self.assertTrue(cl.sizes() == [3, 2, 2])
+
+    def testClusteringOfNullGraph(self):
+        null_graph = Graph()
+        cl = VertexClustering(null_graph, [])
+        self.assertTrue(cl.sizes() == [])
+        self.assertTrue(cl.giant().vcount() == 0)
+        self.assertTrue(cl.giant().ecount() == 0)
+
 
 class CoverTests(unittest.TestCase):
     def setUp(self):
@@ -266,11 +300,11 @@ class CommunityTests(unittest.TestCase):
         cl = g.community_infomap(edge_weights=e_weights, vertex_weights=v_weights)
 
     def testLabelPropagation(self):
-        # Nothing to test there really, since the algorithm
-        # is pretty nondeterministic. We just do a quick smoke
-        # test.
+        # Nothing to test there really, since the algorithm is pretty
+        # nondeterministic. We just do a few quick smoke tests.
         g = Graph.GRG(100, 0.2)
         cl = g.community_label_propagation()
+
         g = Graph([(0, 1), (1, 2), (2, 3)])
         g.es["weight"] = [2, 1, 2]
         g.vs["initial"] = [0, -1, -1, 1]
@@ -283,8 +317,25 @@ class CommunityTests(unittest.TestCase):
             or cl.membership == [0, 0, 0, 1]
         )
 
+        g = Graph.GRG(100, 0.2)
+        g.vs["initial"] = [
+            0 if i == 0 else 1 if i == 99 else 2 if i == 49 else random.randint(0, 50)
+            for i in range(g.vcount())
+        ]
+        g.vs["dont_move"] = [i in (0, 49, 99) for i in range(g.vcount())]
+        cl = g.community_label_propagation(initial="initial", fixed="dont_move")
+
+        # igraph is free to reorder the clusters so only co-membership will be
+        # preserved, hence the next assertion
+        self.assertTrue(
+            cl.membership[0] != cl.membership[49]
+            and cl.membership[49] != cl.membership[99]
+        )
+        self.assertTrue(x >= 0 and x <= 5 for x in cl.membership)
+
     def testMultilevel(self):
         # Example graph from the paper
+        random.seed(42)
         g = Graph(16)
         g += [
             (0, 2),
@@ -316,6 +367,7 @@ class CommunityTests(unittest.TestCase):
             (10, 14),
             (11, 13),
         ]
+
         cls = g.community_multilevel(return_levels=True)
         self.assertTrue(len(cls) == 2)
         self.assertMembershipsEqual(
@@ -327,6 +379,13 @@ class CommunityTests(unittest.TestCase):
         self.assertAlmostEqual(cls[0].q, 0.346301, places=5)
         self.assertAlmostEqual(cls[1].q, 0.392219, places=5)
 
+        cls = g.community_multilevel()
+        self.assertTrue(len(cls.membership) == g.vcount())
+        self.assertMembershipsEqual(
+            cls, [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1]
+        )
+        self.assertAlmostEqual(cls.q, 0.392219, places=5)
+
     def testOptimalModularity(self):
         try:
             g = Graph.Famous("bull")
@@ -457,15 +516,15 @@ class CommunityTests(unittest.TestCase):
 
         import random
 
-        random.seed(0)
+        random.seed(42)
         set_random_number_generator(random)
         # We don't find the optimal partition if we are greedy
         cl = G.community_leiden(
-            "CPM", resolution_parameter=1, weights="weight", beta=0, n_iterations=-1
+            "CPM", resolution=1, weights="weight", beta=0, n_iterations=-1
         )
         self.assertMembershipsEqual(cl, [0, 0, 1, 1, 1, 2, 2, 2])
 
-        random.seed(0)
+        random.seed(42)
         set_random_number_generator(random)
         # We can find the optimal partition if we allow for non-decreasing moves
         # (The randomness is only present in the refinement, which is why we
@@ -473,7 +532,7 @@ class CommunityTests(unittest.TestCase):
         # refined).
         cl = G.community_leiden(
             "CPM",
-            resolution_parameter=1,
+            resolution=1,
             weights="weight",
             beta=5,
             n_iterations=-1,
@@ -605,13 +664,13 @@ class ComparisonTests(unittest.TestCase):
 
 
 def suite():
-    decomposition_suite = unittest.makeSuite(DecompositionTests)
-    clustering_suite = unittest.makeSuite(ClusteringTests)
-    vertex_clustering_suite = unittest.makeSuite(VertexClusteringTests)
-    cover_suite = unittest.makeSuite(CoverTests)
-    community_suite = unittest.makeSuite(CommunityTests)
-    cohesive_blocks_suite = unittest.makeSuite(CohesiveBlocksTests)
-    comparison_suite = unittest.makeSuite(ComparisonTests)
+    decomposition_suite = unittest.defaultTestLoader.loadTestsFromTestCase(DecompositionTests)
+    clustering_suite = unittest.defaultTestLoader.loadTestsFromTestCase(ClusteringTests)
+    vertex_clustering_suite = unittest.defaultTestLoader.loadTestsFromTestCase(VertexClusteringTests)
+    cover_suite = unittest.defaultTestLoader.loadTestsFromTestCase(CoverTests)
+    community_suite = unittest.defaultTestLoader.loadTestsFromTestCase(CommunityTests)
+    cohesive_blocks_suite = unittest.defaultTestLoader.loadTestsFromTestCase(CohesiveBlocksTests)
+    comparison_suite = unittest.defaultTestLoader.loadTestsFromTestCase(ComparisonTests)
     return unittest.TestSuite(
         [
             decomposition_suite,
